<?php

namespace Drupal\migrate_wizard;

use Drupal\Core\Language\LanguageInterface;
use Drupal\user\Entity\Role;
use Drupal\Component\Transliteration\PhpTransliteration;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigManager;
use Drupal\Core\Database\Database;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionList;

/**
 * Defines a Service to update nodes.
 */
class MWManageDataService {

  /**
   * {@inheritdoc}
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected $migrationUrlFilesProd;

  /**
   * {@inheritdoc}
   */
  protected $extensionListModule;

  /**
   * {@inheritdoc}
   */
  protected $configManager;

  /**
   * {@inheritdoc}
   */
  protected $transliterator;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager,
                              EntityTypeManagerInterface $entity_type_manager,
                              ConfigFactoryInterface $configFactory,
                              ExtensionList $extension_list_module,
                              ConfigManager $config_manager,
                              PhpTransliteration $php_transliteration) {

    $this->config = $configFactory->getEditable('migrate_wizard.settings');
    if (is_array($this->config->get('migrate_wizard_settings'))) {
      $this->migrationUrlFilesProd = $this->config->get('migrate_wizard_settings')['migration_url_files_prod'];
    }
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->extensionListModule = $extension_list_module;
    $this->configManager = $config_manager;
    $this->transliterator = $php_transliteration;
  }

  /**
   * {@inheritdoc}
   */
  protected function setUpDatabase() {

    $current_config = $this->config->get('migrate_wizard_settings');
    $key = getenv($current_config["migrate_db_key"]);
    $target = getenv($current_config["migrate_db_target"]);
    $connection = Database::getConnection($target, $key);
    return $connection;
  }

  /**
   * {@inheritdoc}
   */
  protected function getNodeTypesQuery($query) {
    $query = $query->select('node_type', 'nt')
      ->fields('nt', ['type'])
      ->fields('nt', ['name'])
      ->fields('nt', ['description'])
      ->groupBy('nt.type')
      ->groupBy('nt.name')
      ->groupBy('nt.description');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  protected function getVocabulariesQuery($query) {
    $query = $query->select('taxonomy_vocabulary', 'tv')
      ->fields('tv', ['name'])
      ->fields('tv', ['machine_name'])
      ->fields('tv', ['description'])
      ->groupBy('tv.name')
      ->groupBy('tv.machine_name')
      ->groupBy('tv.description');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginNodes() {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_node')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_node')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginNodes() {
    $query = $this->setUpDatabase();
    $query = $this->getNodeTypesQuery($query);
    $results = $query->execute()->fetchAll();
    foreach ($results as $result) {

      // Check if entity exists.
      $entity = $this->entityTypeManager->getStorage('origin_node')->load($result->type);
      if (!$entity) {
        $entity = $this->entityTypeManager->getStorage('origin_node')->create([
          'id' => $result->type,
          'label' => $result->name,
          'description' => $result->description,
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginVocabularies() {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_vocabulary')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginVocabularies() {
    $query = $this->setUpDatabase();
    $query = $this->getVocabulariesQuery($query);
    $results = $query->execute()->fetchAll();
    foreach ($results as $result) {

      // Check if entity exists before create.
      $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->load($result->machine_name);
      if ($entity == NULL) {
        $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->create([
          'id' => $result->machine_name,
          'label' => $result->name,
          'description' => $result->description,
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldsOriginNode($origin_node) {

    $type = 'node';
    if($origin_node == 'user'){
      $type = 'user';
    }
    $query = $this->setUpDatabase();
    $query = $query->select('field_config_instance', 'fci')
      ->fields('fci', ['field_id'])
      ->fields('fci', ['field_name'])
      ->fields('fci', ['data'])
      ->condition('fci.entity_type', $type)
      ->condition('fci.bundle', $origin_node);

    $query->leftJoin('field_config', 'fc', "fci.field_id = fc.id");

    $query->fields("fc", [
      'type',
      'data',
    ]);
    return $query->execute()->fetchAll();
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginDefaultLangcode($current_config = NULL) {
    if($current_config){
      $this->config->set('migrate_wizard_settings', $current_config)->save();
    }

    $query = $this->setUpDatabase();

    $query = $query->select('variable', 'v')
      ->fields('v', ['value'])
      ->condition('v.name', 'language_default');
    $results_query = $query->execute()->fetchAll();
    //unserialize the data
    $results = unserialize($results_query[0]->value);
    return $results->language;
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginLanguages($origin_type, $origin_bundle) {

    $results['origin_languages'] = [];
    if($origin_type == 'node'){
      $query_language_content_type = $this->setUpDatabase();
      $query_language_content_type = $query_language_content_type->select('variable', 'v')
        ->fields('v', ['value'])
        ->condition('v.name', 'language_content_type_'.$origin_bundle);

      $results_language_content_type = $query_language_content_type->execute()->fetchAll();
      $results_language_content_type = reset($results_language_content_type);
      $language_content_type = unserialize($results_language_content_type->value);

      if($language_content_type>0){
        $results['origin_languages'] = $this->getLanguageTranslations();
      }
    }elseif($origin_type == 'vocabulary'){
      $query_type_i18n_mode = $this->setUpDatabase();
      $query_type_i18n_mode = $query_type_i18n_mode->select('taxonomy_vocabulary', 'tv')
        ->fields('tv', ['i18n_mode', 'language'])
        ->condition('tv.machine_name', $origin_bundle);
      $query_type_i18n_mode = $query_type_i18n_mode->execute()->fetchAll();
      $results['i18n_mode'] = $query_type_i18n_mode[0]->i18n_mode;
      $results['language'] = $query_type_i18n_mode[0]->language;
      if($results['i18n_mode'] != 0){
        $results['origin_languages'] = $this->getLanguageTranslations();
      }
    }
    return $results;

  }
  private function getLanguageTranslations(){
    $default_langcode = $this->getOriginDefaultLangcode();
    $query = $this->setUpDatabase();
    $query = $query->select('languages', 'l')
      ->fields('l', ['language'])
      ->condition('l.enabled', '0', '!=')
      ->condition('l.language', $default_langcode, '!=');
    $results_query = $query->execute()->fetchAll();
    $results = [];
    foreach ($results_query as $language) {
      $results[] = $language->language;
    }
    return $results;
  }
  /**
   * {@inheritdoc}
   */
  public function getFieldsOriginVocabulary($origin_vocabulary) {
    $query = $this->setUpDatabase();
    $query = $query->select('taxonomy_vocabulary', 'tv')
      ->fields('fci', ['field_name', 'data'])
      ->condition('tv.machine_name', $origin_vocabulary)
      ->condition('fci.field_name', null, '!=');
    $query->leftJoin('field_config_instance', 'fci', "fci.bundle = tv.machine_name");
    $query->leftJoin('field_config', 'fc', "fc.field_name = fci.field_name");
    $query->fields("fc", [
      'type',
      'data',
    ]);
    return $query->execute()->fetchAll();
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginEditorFormats() {
    $query = $this->setUpDatabase();
    $query = $query->select('ckeditor_input_format', 'cif')
      ->fields('cif', ['name'])
      ->fields('cif', ['format']);
    $results_query = $query->execute()->fetchAll();
    return $results_query;
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentContentTypes() {

    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($content_types as $content_type) {
      $content_types_array[$content_type->id()] = $content_type->label();
    }
    return $content_types_array;
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentVocabularies() {

    // Get all vocabularies.
    $vocabularies = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->loadMultiple();
    foreach ($vocabularies as $vocabularie) {
      $vocabularies_array[$vocabularie->id()] = $vocabularie->label();
    }
    return $vocabularies_array;
  }

  /**
   * {@inheritdoc}
   */
  public function getParagraphBundles() {
    $paragraph_types = $this->entityTypeManager->getStorage('paragraphs_type')->loadMultiple();
    $paragraph_types_array = [];
    foreach ($paragraph_types as $paragraph_type) {
      $paragraph_types_array[$paragraph_type->id()] = $paragraph_type->label();
    }
    return $paragraph_types_array;
  }
  /**
   * {@inheritdoc}
   */
  public function getOriginRoles() {
    $query = $this->setUpDatabase();
    $query = $query->select('role', 'r')
      ->fields('r', ['name'])
      ->fields('r', ['rid']);

    $roles = $query->execute()->fetchAll();

    $roles_array = [];
    foreach ($roles as $rol) {
      $roles_array[$rol->rid] = $rol->name;
    }
    return $roles_array;
  }

  public function getFieldsCollectionByBundle($bundle) {
    $query = $this->setUpDatabase();
    $query = $query->select('field_config_instance', 'fci')
      ->fields('fci', ['field_id'])
      ->fields('fci', ['field_name'])
      ->fields('fci', ['data'])
      ->condition('fci.bundle', $bundle);

    $query->leftJoin('field_config', 'fc', "fci.field_id = fc.id");

    $query->fields("fc", [
      'type',
      'data',
    ]);
    $results = $query->execute()->fetchAll();
    return $results;
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldsMedia() {
    $fieldsArrayMedia = [];
    $media_types = $this->entityTypeManager->getListBuilder('media_type')->load();
    foreach ($media_types as $key => $media_type) {
      $fieldMediaDefinitions = $this->entityFieldManager->getFieldDefinitions('media', $key);
      $fieldMediaBaseDefinitions = $this->entityFieldManager->getBaseFieldDefinitions('media');
      $fieldsExtraMedia = array_diff_key($fieldMediaDefinitions, $fieldMediaBaseDefinitions);
      foreach ($fieldsExtraMedia as $fieldExtraMedia) {
        if ($fieldExtraMedia->getSetting('target_type') == 'file' && !isset($fieldsArrayMedia[$fieldExtraMedia->getName()])) {
          $fieldsArrayMedia[$fieldExtraMedia->getName()] = $fieldExtraMedia->getLabel();
        }
      }
    }
    return $fieldsArrayMedia;
  }

  /**
   * {@inheritdoc}
   */
  public function createRoles($array_config) {
    foreach ($array_config["roles"] as &$role) {
      if ($role['destiny'] == 'create') {
        $data = [
          'id' => $this->transliterateString($role['origin']),
          'label' => $role['origin'],
        ];
        // Creating your role.
        $role_obj = Role::create($data);
        // Saving your role.
        $role_obj->save();
        $role['destiny'] = $role_obj->id();

      }
    }
    $this->config->set('origin_user_roles', $array_config)->save();
  }

  /**
   * Transforms string to machine name format
   *
   * @param string $input
   * @return string|null
   */
  public function transliterateString(string $input)
  {
    $transliterated = \Drupal::transliteration()->transliterate($input, LanguageInterface::LANGCODE_DEFAULT, '_');
    $transliterated = mb_strtolower($transliterated);
    $transliterated = preg_replace('@[^a-z0-9_./\s+/]+@', '_', $transliterated);
    return str_replace(' ', '_', $transliterated);
  }

  public function checkIfCasModuleOriginExistsEnabled(){
    $query = $this->setUpDatabase();
    $query = $query->select('system', 's')
      ->fields('s', ['status'])
      ->condition('s.name', 'cas');
    return $query->execute()->fetchField();
  }
  public function getOriginBundleTitle($bundle, $type){
    $query = $this->setUpDatabase();
    switch($type){
      case 'node':
        $query = $query->select('node_type', 'nt')
          ->fields('nt', ['name'])
          ->condition('nt.type', $bundle);
        break;
      case 'vocabulary':
        $query = $query->select('taxonomy_vocabulary', 'tv')
          ->fields('tv', ['name'])
          ->condition('tv.machine_name', $bundle);
        break;
      case 'user':
        $query = $query->select('role', 'r')
          ->fields('r', ['name'])
          ->condition('r.rid', $bundle);
        break;
    }
    return $query->execute()->fetchField();
  }
  public function checkWorkbenchAccessNodeExist(){
    $query = $this->setUpDatabase();
    //check if table exits
    $query = $query->select('system', 's')
      ->fields('s', ['status'])
      ->condition('s.status', 1)
      ->condition('s.name', 'workbench_access');
    return $query->execute()->fetchField();
  }
}
